@@ -2,7 +2,7 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 
     SPDX-FileCopyrightText: 2002 Oliver Kellogg <okellogg@users.sourceforge.net>
-    SPDX-FileCopyrightText: 2003-2021 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
+    SPDX-FileCopyrightText: 2003-2022 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
 */
 
 #include "adawriter.h"
@@ -58,7 +58,7 @@ Uml::ProgrammingLanguage::Enum AdaWriter::language() const
 /**
  * Return true if `c' is a tagged type or Ada2005 interface.
  */
-bool AdaWriter::isOOClass(UMLClassifier *c)
+bool AdaWriter::isOOClass(const UMLClassifier *c)
 {
     UMLObject::ObjectType ot = c->baseType();
     if (ot == UMLObject::ot_Interface)
@@ -66,7 +66,7 @@ bool AdaWriter::isOOClass(UMLClassifier *c)
     if (ot == UMLObject::ot_Enum)
         return false;
     if (ot != UMLObject::ot_Class) {
-        uDebug() << "unknown object type " << UMLObject::toString(ot);
+        logWarn1("AdaWriter::isOOClass unexpected object type %1", UMLObject::toString(ot));
         return false;
     }
     QString stype = c->stereotype();
@@ -118,7 +118,7 @@ QString AdaWriter::packageName(UMLPackage *p)
     if (umlPkg == UMLApp::app()->document()->rootFolder(Uml::ModelType::Logical))
         umlPkg = 0;
 
-    UMLClassifier *c = p->asUMLClassifier();
+    const UMLClassifier *c = p->asUMLClassifier();
     if (umlPkg == 0) {
         retval = className;
         if (c == 0 || !isOOClass(c))
@@ -207,7 +207,7 @@ void AdaWriter::declareClass(UMLClassifier *c, QTextStream &ada)
 void AdaWriter::writeClass(UMLClassifier *c)
 {
     if (!c) {
-        uDebug() << "Cannot write class of NULL concept!";
+        logWarn0("AdaWriter::writeClass: Cannot write class of NULL concept!");
         return;
     }
     if (m_classesGenerated.contains(c))
@@ -284,7 +284,7 @@ void AdaWriter::writeClass(UMLClassifier *c)
                     // Check whether it's a data type.
                     UMLClassifier *typeObj = t->getType();
                     if (typeObj == 0) {
-                        uError() << "template_param " << typeName << ": typeObj is NULL";
+                        logError1("template_param %1: typeObj is NULL", typeName);
                         ada << indent() << "type " << formalName << " is new " << typeName
                             << " with private;  -- CHECK: codegen error"
                             << m_endl;
