@@ -2,7 +2,7 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 
     SPDX-FileCopyrightText: 2003 Nikolaus Gradwohl <guru@local-guru.net>
-    SPDX-FileCopyrightText: 2004-2021 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
+    SPDX-FileCopyrightText: 2004-2022 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
 */
 
 #include "sqlwriter.h"
@@ -20,6 +20,7 @@
 #include "uniqueconstraint.h"
 #include "umlentityattributelist.h"
 #include "umlclassifierlistitemlist.h"
+#include "uml.h"  // only needed for log{Warn,Error}
 
 #include <KLocalizedString>
 #include <KMessageBox>
@@ -188,7 +189,8 @@ void SQLWriter::writeClass(UMLClassifier *c)
     UMLEntity* e = c->asUMLEntity();
 
     if (!e) {
-        uError() << "Invalid cast from" << c->baseTypeStr() << "'" << c->name() << "' to UMLEntity*";
+        logError2("SQLWriter::writeClass: Invalid cast from '%1' to UMLEntity* %2",
+                  c->name(), c->baseTypeStr());
         return;
     }
 
@@ -234,7 +236,7 @@ void SQLWriter::writeClass(UMLClassifier *c)
     if (language() == Uml::ProgrammingLanguage::PostgreSQL) {
         foreach(UMLEntityAttribute *at, entAttList) {
             if (at->getType()->baseType() == UMLObject::ot_Enum) {
-                UMLEnum *_enum = at->getType()->asUMLEnum();
+                const UMLEnum *_enum = at->getType()->asUMLEnum();
                 if (m_enumsGenerated.contains(at->getTypeName()))
                     continue;
                 m_enumsGenerated.append(at->getTypeName());
@@ -404,7 +406,7 @@ void SQLWriter::printEntityAttributes(QTextStream& sql, UMLEntityAttributeList e
         // the datatype
         if (language() == Uml::ProgrammingLanguage::MySQL &&
                 at->getType() && at->getType()->baseType() == UMLObject::ot_Enum) {
-            UMLEnum *_enum = at->getType()->asUMLEnum();
+            const UMLEnum *_enum = at->getType()->asUMLEnum();
             sql << " ENUM(";
             QString delimiter(QLatin1String(""));
             UMLClassifierListItemList enumLiterals = _enum->getFilteredList(UMLObject::ot_EnumLiteral);
@@ -456,9 +458,10 @@ void SQLWriter::printEntityAttributes(QTextStream& sql, UMLEntityAttributeList e
 void SQLWriter::printUniqueConstraints(QTextStream& sql, UMLClassifierListItemList constrList)
 {
    foreach(UMLClassifierListItem* cli, constrList) {
-       UMLUniqueConstraint* uuc = cli->asUMLUniqueConstraint();
+       const UMLUniqueConstraint* uuc = cli->asUMLUniqueConstraint();
        if (!uuc) {
-           uError() << "Invalid cast from" << cli->baseTypeStr() << "'" << cli->name() << "' to UMLUniqueConstraint*";
+           logError2("SQLWriter::printUniqueConstraints: Invalid cast from '%1' to UMLUniqueConstraint* %2",
+                     cli->name(), cli->baseTypeStr());
            return;
        }
        sql << m_endl;
@@ -505,7 +508,8 @@ void SQLWriter::printForeignKeyConstraints(QTextStream& sql, UMLClassifierListIt
    foreach(UMLClassifierListItem* cli, constrList) {
        UMLForeignKeyConstraint* fkc = cli->asUMLForeignKeyConstraint();
        if (!fkc) {
-           uError() << "Invalid cast from" << cli->baseTypeStr() << "'" << cli->name() << "' to UMLForeignKeyConstraint*";
+           logError2("SQLWriter::printForeignKeyConstraints: Invalid cast from '%1' to UMLForeignKeyConstraint* %2",
+                     cli->name(), cli->baseTypeStr());
            return;
        }
        sql << m_endl;
@@ -630,9 +634,10 @@ void SQLWriter::printAutoIncrements(QTextStream& sql, UMLEntityAttributeList ent
 void SQLWriter::printCheckConstraints(QTextStream& sql, UMLClassifierListItemList constrList)
 {
     foreach(UMLClassifierListItem* cli, constrList) {
-        UMLCheckConstraint* chConstr = cli->asUMLCheckConstraint();
+        const UMLCheckConstraint* chConstr = cli->asUMLCheckConstraint();
         if (!chConstr) {
-            uError() << "Invalid cast from" << cli->baseTypeStr() << "'" << cli->name() << "' to UMLCheckConstraint*";
+            logError2("SQLWriter::printCheckConstraints: Invalid cast from '%1' to UMLCheckConstraint* %2",
+                      cli->name(), cli->baseTypeStr());
             return;
         }
 
