@@ -1,6 +1,6 @@
 /*
     SPDX-License-Identifier: GPL-2.0-or-later
-    SPDX-FileCopyrightText: 2017-2020 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
+    SPDX-FileCopyrightText: 2017-2022 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
 */
 
 // own header
@@ -9,6 +9,7 @@
 // app includes
 #include "associationwidget.h"
 #include "association.h"
+#define DBG_SRC QLatin1String("Diagram_Utils")
 #include "debug_utils.h"
 #include "import_utils.h"
 #include "messagewidget.h"
@@ -27,9 +28,8 @@
 #include <QMimeData>
 #include <QRegExp>
 
+// Currently this file is not using debug statements. Activate this line when inserting them:
 DEBUG_REGISTER_DISABLED(Diagram_Utils)
-#undef DBG_SRC
-#define DBG_SRC QLatin1String("Diagram_Utils")
 
 namespace Diagram_Utils {
 
@@ -77,6 +77,7 @@ SequenceLineFormat detectSequenceLineFormat(const QStringList &lines)
  * @param sequence return of sequence number
  * @param package return of package
  * @param method return of method
+ * @param error  return of error string if error happened
  * @return true line could be parsed and return variable has been filled
  * @return false line could not be parsed, no return variable has been filled
  */
@@ -238,7 +239,7 @@ bool importSequences(const QStringList &lines, UMLScene *scene, const QString &f
                 QString item = QString::fromLatin1("%1:%2:%3: %4: %5")
                         .arg(fileName).arg(index)
                         .arg(1).arg(line).arg(error);
-                UMLApp::app()->logWindow()->addItem(item);
+                UMLApp::app()->log(item);
             }
             continue;
         }
@@ -407,12 +408,11 @@ bool importGraph(const QStringList &lines, UMLScene *scene, const QString &fileN
                 AssociationWidget* aw = AssociationWidget::create(scene, w1, type, w2, assoc);
                 scene->addAssociation(aw);
             } else {
-                if (assoc)
-                    delete assoc;
+                // in case of error, assoc remains nullptr
                 QString item = QString::fromLatin1("%1:%2:%3: %4: %5")
                         .arg(fileName).arg(lineNumber)
                         .arg(1).arg(line).arg(QLatin1String("error:could not add association"));
-                UMLApp::app()->logWindow()->addItem(item);
+                UMLApp::app()->log(item);
             }
         } else if (l[0].isEmpty() && c && l.size() == 2) {
             QString name = l.last();
@@ -444,7 +444,7 @@ bool importGraph(const QStringList &lines, UMLScene *scene, const QString &fileN
             QString item = QString::fromLatin1("%1:%2:%3: %4: %5")
                     .arg(fileName).arg(lineNumber)
                     .arg(1).arg(line).arg(QLatin1String("syntax error"));
-            UMLApp::app()->logWindow()->addItem(item);
+            UMLApp::app()->log(item);
         }
     }
     return true;
