@@ -1,6 +1,6 @@
 /*
     SPDX-License-Identifier: GPL-2.0-or-later
-    SPDX-FileCopyrightText: 2004-2021 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
+    SPDX-FileCopyrightText: 2004-2022 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
 */
 
 // own header
@@ -8,6 +8,7 @@
 
 // app includes
 #include "floatingtextwidget.h"
+#define DBG_SRC QLatin1String("Model_Utils")
 #include "debug_utils.h"
 #include "umlobject.h"
 #include "umlpackagelist.h"
@@ -38,6 +39,8 @@
 #include <QRegExp>
 #include <QStringList>
 
+DEBUG_REGISTER(Model_Utils)
+
 namespace Model_Utils {
 
 /**
@@ -68,6 +71,36 @@ bool isCloneable(WidgetBase::WidgetType type)
     }
 }
 
+/**
+ * Normalize a type name with respect to interspersed spaces.
+ * @param type  Input type name e.g. from a user text entry dialog.
+ * @return      Normalized type name.
+ */
+QString normalize(QString type)
+{
+    QString str = type.simplified();
+    int pos;
+    // Remove space between word and non word
+    QRegExp word_nonword(QLatin1String("\\w \\W"));
+    pos = 0;
+    while ((pos = word_nonword.indexIn(str, pos)) != -1) {
+        str.remove(++pos, 1);
+    }
+    // Remove space between non word and word
+    QRegExp nonword_word(QLatin1String("\\W \\w"));
+    pos = 0;
+    while ((pos = nonword_word.indexIn(str, pos)) != -1) {
+        str.remove(++pos, 1);
+    }
+    // Remove space between non word and non word
+    QRegExp nonword_nonword(QLatin1String("\\W \\W"));
+    pos = 0;
+    while ((pos = nonword_nonword.indexIn(str, pos)) != -1) {
+        str.remove(++pos, 1);
+    }
+    return str;
+}
+
 /**
  * Seek the given id in the given list of objects.
  * Each list element may itself contain other objects
@@ -155,14 +188,14 @@ UMLObject* findUMLObject(const UMLObjectList& inList,
                          UMLObject *currentObj /* = 0 */)
 {
     const bool caseSensitive = UMLApp::app()->activeLanguageIsCaseSensitive();
-    QString name = inName;
+    QString name = normalize(inName);
     const bool atGlobalScope = name.startsWith(QLatin1String("::"));
     if (atGlobalScope) {
         name = name.mid(2);
         currentObj = 0;
     }
     QStringList components;
-#ifdef TRY_BUGFIX_120682
+#ifdef TRY_BUGFIX_120862
     // If we have a pointer or a reference in cpp we need to remove
     // the asterisks and ampersands in order to find the appropriate object
     if (UMLApp::app()->getActiveLanguage() == Uml::pl_Cpp) {
@@ -204,9 +237,8 @@ UMLObject* findUMLObject(const UMLObjectList& inList,
                 }
             }
             if (seenPkgs.indexOf(pkg) != -1) {
-                uError() << "findUMLObject(" << name << "): "
-                    << "breaking out of cycle involving "
-                    << pkg->name();
+                logError2("Model_Utils::findUMLObject(%1): breaking out of cycle involving %2",
+                          name, pkg->name());
                 break;
             }
             seenPkgs.append(pkg);
@@ -235,10 +267,8 @@ UMLObject* findUMLObject(const UMLObjectList& inList,
                 UMLObject::ObjectType foundType = obj->baseType();
                 if (nameWithoutFirstPrefix.isEmpty()) {
                     if (type != UMLObject::ot_UMLObject && type != foundType) {
-                        uDebug() << "type mismatch for "
-                            << name << " (seeking type: "
-                            << UMLObject::toString(type) << ", found type: "
-                            << UMLObject::toString(foundType) << ")";
+                        logDebug3("findUMLObject type mismatch for %1 (seeking type: %2, found type: %3)",
+                                  name, UMLObject::toString(type), UMLObject::toString(foundType));
                         // Class, Interface, and Datatype are all Classifiers
                         // and are considered equivalent.
                         // The caller must be prepared to handle possible mismatches.
@@ -264,8 +294,7 @@ UMLObject* findUMLObject(const UMLObjectList& inList,
                     foundType != UMLObject::ot_Class &&
                     foundType != UMLObject::ot_Interface &&
                     foundType != UMLObject::ot_Component) {
-                    uDebug() << "found " << UMLObject::toString(foundType) << name
-                             << " is not a package (?)";
+                    logDebug2("findUMLObject found %1 %2 is not a package (?)", UMLObject::toString(foundType), name);
                     continue;
                 }
                 UMLPackage *pkg = obj->asUMLPackage();
@@ -292,10 +321,8 @@ UMLObject* findUMLObject(const UMLObjectList& inList,
                     && obj->stereotype() == QLatin1String("class-or-package")) {
                     return obj;
                 }
-                uDebug() << "type mismatch for "
-                    << name << " (seeking type: "
-                    << UMLObject::toString(type) << ", found type: "
-                    << UMLObject::toString(foundType) << ")";
+                logDebug3("findUMLObject type mismatch for %1 (seeking type: %2, found type: %3)",
+                          name, UMLObject::toString(type), UMLObject::toString(foundType));
                 continue;
             }
             return obj;
@@ -305,11 +332,10 @@ UMLObject* findUMLObject(const UMLObjectList& inList,
             foundType != UMLObject::ot_Class &&
             foundType != UMLObject::ot_Interface &&
             foundType != UMLObject::ot_Component) {
-            uDebug() << "found " << name << "(" << UMLObject::toString(foundType) << ")"
-                     << " is not a package (?)";
+            logDebug2("findUMLObject found %1 (%2) is not a package (?)", name, UMLObject::toString(foundType));
             continue;
         }
-        UMLPackage *pkg = obj->asUMLPackage();
+        const UMLPackage *pkg = obj->asUMLPackage();
         return findUMLObject(pkg->containedObjects(),
                               nameWithoutFirstPrefix, type);
     }
@@ -364,7 +390,7 @@ UMLObject* findUMLObjectRecursive(const UMLObjectList& inList,
     foreach(UMLObject *obj, inList) {
         if (obj->name() == name && type == obj->baseType())
             return obj;
-        UMLPackage *pkg = obj->asUMLPackage();
+        const UMLPackage *pkg = obj->asUMLPackage();
         if (pkg && pkg->containedObjects().size() > 0) {
             UMLObject *o = findUMLObjectRecursive(pkg->containedObjects(), name, type);
             if (o)
@@ -450,7 +476,9 @@ void treeViewMoveObjectTo(UMLObject* container, UMLObject* object)
 UMLObject* treeViewGetCurrentObject()
 {
     UMLListView *listView = UMLApp::app()->listView();
-    UMLListViewItem *current = static_cast<UMLListViewItem*>(listView->currentItem());
+    UMLListViewItem *current = dynamic_cast<UMLListViewItem*>(listView->currentItem());
+    if (current == nullptr)
+        return nullptr;
     return current->umlObject();
 }
 
@@ -508,7 +536,8 @@ QString treeViewBuildDiagramName(Uml::ID::Type id)
         return name;
     }
     else {
-        uWarning() << "diagram not found - returning empty name!";
+        logWarn1("Model_Utils::treeViewBuildDiagramName: diagram with id %1 not found",
+                 Uml::ID::toString(id));
         return QString();
     }
 }
@@ -558,7 +587,7 @@ QString uniqObjectName(UMLObject::ObjectType type, UMLPackage *parentPkg, QStrin
         case UMLObject::ot_UseCase:             currentName = i18n("new_use case");               break;
         default:
             currentName = i18n("new_object");
-            uWarning() << "model_utils::uniqObjectName unknown object type" << UMLObject::toString(type);
+            logWarn1("Model_Utils::uniqObjectName unknown object type %1", UMLObject::toString(type));
         }
     }
     UMLDoc *doc = UMLApp::app()->document();
@@ -606,7 +635,7 @@ QString newTitle(UMLObject::ObjectType type)
     case UMLObject::ot_UniqueConstraint:    return i18n("New unique constraint");
     case UMLObject::ot_UseCase:             return i18n("New use case");
     default:
-        uWarning() << "model_utils::newTitle unknown object type" << UMLObject::toString(type);
+        logWarn1("Model_Utils::newTitle unknown object type %1", UMLObject::toString(type));
         return i18n("New UML object");
     }
 }
@@ -648,7 +677,7 @@ QString newText(UMLObject::ObjectType type)
     case UMLObject::ot_UniqueConstraint:    return i18n("Enter the name of the new unique constraint:");
     case UMLObject::ot_UseCase:             return i18n("Enter the name of the new use case:");
     default:
-        uWarning() << "model_utils::newText unknown object type" << UMLObject::toString(type);
+        logWarn1("Model_utilS::newText unknown object type %1", UMLObject::toString(type));
         return i18n("Enter the name of the new UML object");
     }
 }
@@ -690,7 +719,7 @@ QString renameTitle(UMLObject::ObjectType type)
     case UMLObject::ot_UniqueConstraint:    return i18n("Rename unique constraint");
     case UMLObject::ot_UseCase:             return i18n("Rename use case");
     default:
-        uWarning() << "model_utils::renameTitle unknown object type" << UMLObject::toString(type);
+        logWarn1("Model_Utils::renameTitle unknown object type %1", UMLObject::toString(type));
         return i18n("Rename UML object");
     }
 }
@@ -732,7 +761,7 @@ QString renameText(UMLObject::ObjectType type)
     case UMLObject::ot_UniqueConstraint:    return i18n("Enter the new name of the unique constraint:");
     case UMLObject::ot_UseCase:             return i18n("Enter the new name of the use case:");
     default:
-        uWarning() << "model_utils::renameText unknown object type" << UMLObject::toString(type);
+        logWarn1("Model_Utils::renameText unknown object type %1", UMLObject::toString(type));
         return i18n("Enter the new name of the UML object");
     }
 }
@@ -742,9 +771,9 @@ QString renameText(UMLObject::ObjectType type)
  */
 QString getXmiId(QDomElement element)
 {
-    QString idStr = element.attribute(QLatin1String("xmi.id"));
+    QString idStr = element.attribute(QLatin1String("xmi:id"));
     if (idStr.isEmpty())
-        idStr = element.attribute(QLatin1String("xmi:id"));
+        idStr = element.attribute(QLatin1String("xmi.id"));
     return idStr;
 }
 
@@ -1437,7 +1466,7 @@ Uml::ModelType::Enum convert_DT_MT(Uml::DiagramType::Enum dt)
             mt = Uml::ModelType::EntityRelationship;
             break;
         default:
-            uError() << "Model_Utils::convert_DT_MT: illegal input value " << dt;
+            logError1("Model_Utils::convert_DT_MT: illegal input value %1", dt);
             mt = Uml::ModelType::N_MODELTYPES;
             break;
     }
@@ -1550,7 +1579,7 @@ UMLListViewItem::ListViewType convert_DT_LVT(Uml::DiagramType::Enum dt)
         break;
 
     default:
-        uWarning() << "convert_DT_LVT() called on unknown diagram type";
+        logWarn1("Model_Utils::convert_DT_LVT() called on unknown diagram type %1", dt);
     }
     return type;
 }
@@ -1619,8 +1648,8 @@ UMLListViewItem::ListViewType convert_OT_LVT(UMLObject *o)
                     return type;
                 }
             } while ((p = p->umlPackage()) != 0);
-            uError() << "convert_OT_LVT(" << o->name()
-                << "): internal error - object is not properly nested in folder";
+            logError1("Model_Utils::convert_OT_LVT(%1): internal error - "
+                      "object is not properly nested in folder", o->name());
         }
         break;
 
@@ -1669,8 +1698,8 @@ UMLListViewItem::ListViewType convert_OT_LVT(UMLObject *o)
         break;
 
     case UMLObject::ot_UniqueConstraint: {
-         UMLEntity* ent = o->umlParent()->asUMLEntity();
-         UMLUniqueConstraint* uc = o->asUMLUniqueConstraint();
+         const UMLEntity* ent = o->umlParent()->asUMLEntity();
+         const UMLUniqueConstraint* uc = o->asUMLUniqueConstraint();
          if (ent->isPrimaryKey(uc)) {
              type = UMLListViewItem::lvt_PrimaryKeyConstraint;
          } else {
@@ -1837,6 +1866,9 @@ UMLObject::ObjectType convert_LVT_OT(UMLListViewItem::ListViewType lvt)
  * Return the IconType which corresponds to the given listview type.
  *
  * @param lvt  ListViewType to convert.
+ * @param o    Optional UMLObject pointer is only used if @p lvt is lvt_Class:
+ *             If the stereotype <<class-or-package>> is applied on the object
+ *             then Icon_Utils::it_ClassOrPackage is returned.
  * @return  The Icon_Utils::IconType corresponding to the lvt.
  *          Returns it_Home in case no mapping to IconType exists.
  */
