@@ -1,11 +1,14 @@
 /*
     SPDX-License-Identifier: GPL-2.0-or-later
-    SPDX-FileCopyrightText: 2002-2021 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
+    SPDX-FileCopyrightText: 2002-2022 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
 */
 
 // own header
 #include "umllistviewitem.h"
 
+// definition required by debug_utils.h
+#define DBG_SRC QLatin1String("UMLListViewItem")
+
 // app includes
 #include "debug_utils.h"
 #include "folder.h"
@@ -40,8 +43,6 @@
 // system includes
 #include <cstdlib>
 
-#define DBG_LVI QLatin1String("UMLListViewItem")
-
 DEBUG_REGISTER(UMLListViewItem)
 
 /**
@@ -58,7 +59,7 @@ UMLListViewItem::UMLListViewItem(UMLListView * parent, const QString &name,
 {
     init();
     if (parent == 0) {
-        DEBUG(DBG_LVI) << "UMLListViewItem constructor called with a null listview parent";
+        logDebug0("UMLListViewItem constructor called with a null listview parent");
     }
     m_type = t;
     m_object = o;
@@ -70,7 +71,7 @@ UMLListViewItem::UMLListViewItem(UMLListView * parent, const QString &name,
 }
 
 /**
- * Sets up an instance for subsequent loadFromXMI1().
+ * Sets up an instance for subsequent loadFromXMI().
  *
  * @param parent   The parent to this instance.
  */
@@ -79,12 +80,12 @@ UMLListViewItem::UMLListViewItem(UMLListView * parent)
 {
     init();
     if (parent == 0) {
-        DEBUG(DBG_LVI) << "UMLListViewItem constructor called with a NULL listview parent";
+        logDebug0("UMLListViewItem constructor called with a NULL listview parent");
     }
 }
 
 /**
- * Sets up an instance for subsequent loadFromXMI1().
+ * Sets up an instance for subsequent loadFromXMI().
  *
  * @param parent   The parent to this instance.
  */
@@ -204,12 +205,12 @@ QString UMLListViewItem::toolTip() const
                 return obj->doc();
             case UMLObject::ot_Operation:
             {
-                UMLOperation *op = obj->asUMLOperation();
+                const UMLOperation *op = obj->asUMLOperation();
                 return op->toString(Uml::SignatureType::ShowSig);
             }
             case UMLObject::ot_Attribute:
             {
-                UMLAttribute *at = obj->asUMLAttribute();
+                const UMLAttribute *at = obj->asUMLAttribute();
                 return at->toString(Uml::SignatureType::ShowSig);
             }
             default:
@@ -237,7 +238,7 @@ UMLListViewItem::ListViewType UMLListViewItem::type() const
 void UMLListViewItem::addChildItem(UMLObject *child, UMLListViewItem *childItem)
 {
     if (!child) {
-        uError() << "UMLListViewItem::addChildItem called with null child";
+        logError0("UMLListViewItem::addChildItem called with null child");
         return;
     }
     m_comap[child] = childItem;
@@ -249,12 +250,12 @@ void UMLListViewItem::addChildItem(UMLObject *child, UMLListViewItem *childItem)
 void UMLListViewItem::deleteChildItem(UMLObject *child)
 {
     if (!child) {
-        uError() << "UMLListViewItem::deleteChildItem called with null child";
+        logError0("UMLListViewItem::deleteChildItem called with null child");
         return;
     }
     UMLListViewItem *childItem = findChildObject(child);
     if (childItem == 0) {
-        uError() << child->name() << ": child listview item not found";
+        logError1("UMLListViewItem::deleteChildItem: child listview item %1 not found", child->name());
         return;
     }
     m_comap.remove(child);
@@ -290,8 +291,8 @@ void UMLListViewItem::setID(Uml::ID::Type id)
     if (m_object) {
         Uml::ID::Type oid = m_object->id();
         if (id != Uml::ID::None && oid != id) {
-            DEBUG(DBG_LVI) << "new id " << Uml::ID::toString(id) << " does not agree with object id "
-                << Uml::ID::toString(oid);
+            logDebug2("UMLListViewItem::setID: new id %1 does not agree with object id %2",
+                      Uml::ID::toString(id), Uml::ID::toString(oid));
         }
     }
     m_id = id;
@@ -326,7 +327,8 @@ bool UMLListViewItem::isOwnParent(Uml::ID::Type listViewItemID)
     UMLListView* listView = static_cast<UMLListView*>(treeWidget());
     QTreeWidgetItem *lvi = static_cast<QTreeWidgetItem*>(listView->findItem(listViewItemID));
     if (lvi == 0) {
-        uError() << "ListView->findItem(" << Uml::ID::toString(listViewItemID) << ") returns NULL";
+        logError1("UMLListViewItem::isOwnParent: listView->findItem(%1) returns null",
+                  Uml::ID::toString(listViewItemID));
         return true;
     }
     for (QTreeWidgetItem *self = static_cast<QTreeWidgetItem*>(this); self; self = self->parent()) {
@@ -359,10 +361,10 @@ void UMLListViewItem::updateObject()
     UMLObject::ObjectType ot = m_object->baseType();
     QString modelObjText = m_object->name();
     if (Model_Utils::isClassifierListitem(ot)) {
-        UMLClassifierListItem *pNarrowed = m_object->asUMLClassifierListItem();
+        const UMLClassifierListItem *pNarrowed = m_object->asUMLClassifierListItem();
         modelObjText = pNarrowed->toString(Uml::SignatureType::SigNoVis);
     } else if (ot == UMLObject::ot_InstanceAttribute) {
-        UMLInstanceAttribute *pNarrowed = m_object->asUMLInstanceAttribute();
+        const UMLInstanceAttribute *pNarrowed = m_object->asUMLInstanceAttribute();
         modelObjText = pNarrowed->toString();
     }
     setText(modelObjText);
@@ -487,7 +489,7 @@ void UMLListViewItem::slotEditFinished(const QString &newText)
 {
     m_label = text(0);
 
-    DEBUG(DBG_LVI) << this << "text=" << newText;
+    logDebug1("UMLListViewItem::slotEditFinished: text=%1", newText);
     UMLListView* listView = static_cast<UMLListView*>(treeWidget());
     UMLDoc* doc = listView->document();
     if (newText == m_label) {
@@ -713,7 +715,7 @@ void UMLListViewItem::slotEditFinished(const QString &newText)
  */
 void UMLListViewItem::cancelRenameWithMsg()
 {
-    DEBUG(DBG_LVI) << this << " - column=" << ":TODO:col" << ", text=" << text(0);
+    logDebug1("UMLListViewItem::cancelRenameWithMsg - column=:TODO:col, text=%1", text(0));
     KMessageBox::error(0,
                        i18n("The name you entered was invalid.\nRenaming process has been canceled."),
                        i18n("Name Not Valid"));
@@ -747,14 +749,14 @@ int UMLListViewItem::compare(QTreeWidgetItem *other, int col, bool ascending) co
     if (m_object == 0) {
         retval = (subItem ? 1 : alphaOrder);
 #ifdef DEBUG_LVITEM_INSERTION_ORDER
-        DEBUG(DBG_LVI) << dbgPfx << retval << " because (m_object==0)";
+        logDebug2("UMLListViewItem::%1 %2 because (m_object==0)", dbgPfx, retval);
 #endif
         return retval;
     }
     if (otherObj == 0) {
         retval = (subItem ? -1 : alphaOrder);
 #ifdef DEBUG_LVITEM_INSERTION_ORDER
-        DEBUG(DBG_LVI) << dbgPfx << retval << " because (otherObj==0)";
+        logDebug2("UMLListViewItem::%1 %2 because (otherObj==0)", dbgPfx, retval);
 #endif
         return retval;
     }
@@ -763,21 +765,22 @@ int UMLListViewItem::compare(QTreeWidgetItem *other, int col, bool ascending) co
     if (ourParent == 0) {
         retval = (subItem ? 1 : alphaOrder);
 #ifdef DEBUG_LVITEM_INSERTION_ORDER
-        DEBUG(DBG_LVI) << dbgPfx << retval << " because (ourParent==0)";
+        logDebug2("UMLListViewItem::%1 %2 because (ourParent==0)", dbgPfx, retval);
 #endif
         return retval;
     }
     if (otherParent == 0) {
         retval = (subItem ? -1 : alphaOrder);
 #ifdef DEBUG_LVITEM_INSERTION_ORDER
-        DEBUG(DBG_LVI) << dbgPfx << retval << " because (otherParent==0)";
+        logDebug2("UMLListViewItem::%1 %2 because (otherParent==0)", dbgPfx, retval);
 #endif
         return retval;
     }
     if (ourParent != otherParent) {
         retval = (subItem ? 0 : alphaOrder);
 #ifdef DEBUG_LVITEM_INSERTION_ORDER
-        DEBUG(DBG_LVI) << dbgPfx << retval << " because (ourParent != otherParent)";
+        logDebug2("UMLListViewItem::%1 %2 because (ourParent != otherParent)",
+                  dbgPfx, retval);
 #endif
         return retval;
     }
@@ -786,14 +789,14 @@ int UMLListViewItem::compare(QTreeWidgetItem *other, int col, bool ascending) co
     if (thisUmlItem == 0) {
         retval = (subItem ? 1 : alphaOrder);
 #ifdef DEBUG_LVITEM_INSERTION_ORDER
-        DEBUG(DBG_LVI) << dbgPfx << retval << " because (thisUmlItem==0)";
+        logDebug2("UMLListViewItem::%1 %2 because (thisUmlItem==0)", dbgPfx, retval);
 #endif
         return retval;
     }
     if (otherUmlItem == 0) {
         retval = (subItem ? -1 : alphaOrder);
 #ifdef DEBUG_LVITEM_INSERTION_ORDER
-        DEBUG(DBG_LVI) << dbgPfx << retval << " because (otherUmlItem==0)";
+        logDebug2("UMLListViewItem::%1 %2 because (otherUmlItem==0)", dbgPfx, retval);
 #endif
         return retval;
     }
@@ -802,12 +805,12 @@ int UMLListViewItem::compare(QTreeWidgetItem *other, int col, bool ascending) co
     int otherIndex = items.indexOf(otherUmlItem);
     if (myIndex < 0) {
         retval = (subItem ? -1 : alphaOrder);
-        uError() << dbgPfx << retval << " because (myIndex < 0)";
+        logError2("UMLListViewItem::%1 %2 because (myIndex < 0)", dbgPfx, retval);
         return retval;
     }
     if (otherIndex < 0) {
         retval = (subItem ? 1 : alphaOrder);
-        uError() << dbgPfx << retval << " because (otherIndex < 0)";
+        logError2("UMLListViewItem::%1 %2 because (otherIndex < 0)", dbgPfx, retval);
         return retval;
     }
     return (myIndex < otherIndex ? -1 : myIndex > otherIndex ? 1 : 0);
@@ -894,30 +897,31 @@ UMLListViewItem * UMLListViewItem::findItem(Uml::ID::Type id)
 /**
  * Saves the listview item to a "listitem" tag.
  */
-void UMLListViewItem::saveToXMI1(QXmlStreamWriter& writer)
+void UMLListViewItem::saveToXMI(QXmlStreamWriter& writer)
 {
     writer.writeStartElement(QLatin1String("listitem"));
     Uml::ID::Type id = ID();
     QString idStr = Uml::ID::toString(id);
-    //DEBUG(DBG_LVI) << "id = " << idStr << ", type = " << m_type;
+    //logDebug2("UMLListViewItem::saveToXMI: id = %1, type =%2", idStr, m_type);
     if (id != Uml::ID::None)
         writer.writeAttribute(QLatin1String("id"), idStr);
     writer.writeAttribute(QLatin1String("type"), QString::number(m_type));
     if (m_object == 0) {
         if (! Model_Utils::typeIsDiagram(m_type) && m_type != lvt_View)
-            uError() << text(0) << ": m_object is NULL";
+            logError1("UMLListViewItem::saveToXMI(%1) : m_object is NULL", text(0));
         if (m_type != lvt_View)
             writer.writeAttribute(QLatin1String("label"), text(0));
     } else if (m_object->id() == Uml::ID::None) {
         if (text(0).isEmpty()) {
-            DEBUG(DBG_LVI) << "Skipping empty item";
+            logDebug0("UMLListViewItem::saveToXMI(: Skipping empty item");
             return;
         }
-        DEBUG(DBG_LVI) << "saving local label " << text(0) << " because umlobject ID is not set";
+        logDebug1("UMLListViewItem::saveToXMI saving local label %1 because umlobject ID is not set",
+                  text(0));
         if (m_type != lvt_View)
             writer.writeAttribute(QLatin1String("label"), text(0));
     } else if (m_object->baseType() == UMLObject::ot_Folder) {
-        UMLFolder *extFolder = m_object->asUMLFolder();
+        const UMLFolder *extFolder = m_object->asUMLFolder();
         if (!extFolder->folderFile().isEmpty()) {
             writer.writeAttribute(QLatin1String("open"), QLatin1String("0"));
             writer.writeEndElement();
@@ -927,7 +931,7 @@ void UMLListViewItem::saveToXMI1(QXmlStreamWriter& writer)
     writer.writeAttribute(QLatin1String("open"), QString::number(isExpanded()));
     for (int i = 0; i < childCount(); i++) {
         UMLListViewItem *childItem = static_cast<UMLListViewItem*>(child(i));
-        childItem->saveToXMI1(writer);
+        childItem->saveToXMI(writer);
     }
     writer.writeEndElement();
 }
@@ -935,7 +939,7 @@ void UMLListViewItem::saveToXMI1(QXmlStreamWriter& writer)
 /**
  * Loads a "listitem" tag, this is only used by the clipboard currently.
  */
-bool UMLListViewItem::loadFromXMI1(QDomElement& qElement)
+bool UMLListViewItem::loadFromXMI(QDomElement& qElement)
 {
     QString id = qElement.attribute(QLatin1String("id"), QLatin1String("-1"));
     QString type = qElement.attribute(QLatin1String("type"), QLatin1String("-1"));
@@ -944,7 +948,7 @@ bool UMLListViewItem::loadFromXMI1(QDomElement& qElement)
     if (!label.isEmpty())
         setText(label);
     else if (id == QLatin1String("-1")) {
-        uError() << "Item of type " << type << " has neither ID nor label";
+        logError1("UMLListViewItem::saveToXMI: Item of type %1 has neither ID nor label", type);
         return false;
     }
 
