@@ -1,12 +1,7 @@
-/***************************************************************************
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   copyright (C) 2002-2014                                               *
- *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
- ***************************************************************************/
+/*
+    SPDX-License-Identifier: GPL-2.0-or-later
+    SPDX-FileCopyrightText: 2002-2022 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
+*/
 #include "classifier.h"
 
 // app includes
@@ -32,6 +27,9 @@
 #include "umltemplatedialog.h"
 #include "optionstate.h"
 #include "icon_utils.h"
+#include "instance.h"
+#include "instanceattribute.h"
+#include "optionstate.h"
 
 // kde includes
 #include <KLocalizedString>
@@ -42,10 +40,12 @@
 
 using namespace Uml;
 
+DEBUG_REGISTER(UMLClassifier)
+
 /**
  * @brief holds set of classifiers for recursive loop detection
  */
-class UMLClassifierSet: public QSet<UMLClassifier *> {
+class UMLClassifierSet: public QSet<const UMLClassifier *> {
 public:
     UMLClassifierSet() : level(0)
     {
@@ -104,7 +104,7 @@ void UMLClassifier::setBaseType(UMLObject::ObjectType ot)
             newIcon = Icon_Utils::it_Package;
             break;
         default:
-            uError() << "cannot set to type " << ot;
+            logError2("UMLClassifier %1 setBaseType : cannot set to type %2", name(), ot);
             return;
     }
     Model_Utils::treeViewChangeIcon(this, newIcon);
@@ -132,7 +132,7 @@ bool UMLClassifier::isInterface() const
 UMLOperation * UMLClassifier::checkOperationSignature(
         const QString& name,
         UMLAttributeList opParams,
-        UMLOperation *exemptOp)
+        UMLOperation *exemptOp) const
 {
     UMLOperationList list = findOperations(name);
     if (list.count() == 0) {
@@ -175,7 +175,7 @@ UMLOperation * UMLClassifier::checkOperationSignature(
  * @return  The operation found.  Will return 0 if none found.
  */
 UMLOperation* UMLClassifier::findOperation(const QString& name,
-                                           Model_Utils::NameAndType_List params)
+                                           Model_Utils::NameAndType_List params) const
 {
     UMLOperationList list = findOperations(name);
     if (list.count() == 0) {
@@ -260,6 +260,11 @@ UMLOperation* UMLClassifier::createOperation(
         }
     }
 
+    if (isInterface()) {
+        op->setAbstract(true);
+        op->setVirtual(true);
+    }
+
     // Only show the operation dialog if no name was provided (allows quick-create
     // from listview)
     if (nameNotSet) {
@@ -303,27 +308,28 @@ UMLOperation* UMLClassifier::createOperation(
 bool UMLClassifier::addOperation(UMLOperation* op, int position)
 {
     Q_ASSERT(op);
-    if (m_List.indexOf(op) != -1) {
-        uDebug() << "findRef(" << op->name() << ") finds op (bad)";
+    if (subordinates().indexOf(op) != -1) {
+        logDebug1("UMLClassifier::addOperation: findRef(%1) finds op (bad)", op->name());
         return false;
     }
     if (checkOperationSignature(op->name(), op->getParmList())) {
-        uDebug() << "checkOperationSignature(" << op->name() << ") op is non-unique";
+        logDebug1("UMLClassifier::addOperation: checkOperationSignature(%1) op is non-unique",
+                  op->name());
         return false;
     }
 
-    if (position >= 0 && position <= m_List.count()) {
-        uDebug() << op->name() << ": inserting at position " << position;
-        m_List.insert(position, op);
+    if (position >= 0 && position <= subordinates().count()) {
+        logDebug2("UMLClassifier::addOperation(%1): inserting at position %2", op->name(), position);
+        subordinates().insert(position, op);
         UMLClassifierListItemList itemList = getFilteredList(UMLObject::ot_Operation);
         QString buf;
         foreach (UMLClassifierListItem* currentAtt, itemList) {
             buf.append(QLatin1Char(' ') + currentAtt->name());
         }
-        uDebug() << "  list after change: " << buf;
+        logDebug1("UMLClassifier::addOperation list after change:%1", buf);
     }
     else {
-        m_List.append(op);
+        subordinates().append(op);
     }
     emit operationAdded(op);
     UMLObject::emitModified();
@@ -363,11 +369,11 @@ bool UMLClassifier::addOperation(UMLOperation* op, IDChangeLog* log)
 int UMLClassifier::removeOperation(UMLOperation *op)
 {
     if (op == 0) {
-        uDebug() << "called on NULL op";
+        logDebug0("UMLClassifier::removeOperation called on NULL op");
         return -1;
     }
-    if (!m_List.removeAll(op)) {
-        uDebug() << "cannot find op " << op->name() << " in list";
+    if (!subordinates().removeAll(op)) {
+        logDebug1("UMLClassifier::removeOperation cannot find op %1 in list", op->name());
         return -1;
     }
     // disconnection needed.
@@ -375,7 +381,7 @@ int UMLClassifier::removeOperation(UMLOperation *op)
     disconnect(op, SIGNAL(modified()), this, SIGNAL(modified()));
     emit operationRemoved(op);
     UMLObject::emitModified();
-    return m_List.count();
+    return subordinates().count();
 }
 
 /**
@@ -422,17 +428,6 @@ UMLObject* UMLClassifier::createTemplate(const QString& currentName /*= QString(
     return newTemplate;
 }
 
-/**
- * Returns the number of attributes for the class.
- *
- * @return  The number of attributes for the class.
- */
-int UMLClassifier::attributes()
-{
-    UMLClassifierListItemList atts = getFilteredList(UMLObject::ot_Attribute);
-    return atts.count();
-}
-
 /**
  * Returns the attributes for the specified scope.
  * @return   List of true attributes for the class.
@@ -440,8 +435,7 @@ int UMLClassifier::attributes()
 UMLAttributeList UMLClassifier::getAttributeList() const
 {
     UMLAttributeList attributeList;
-    foreach (UMLObject* listItem, m_List) {
-        uIgnoreZeroPointer(listItem);
+    foreach (UMLObject* listItem, subordinates()) {
         if (listItem->baseType() == UMLObject::ot_Attribute) {
             attributeList.append(listItem->asUMLAttribute());
         }
@@ -524,12 +518,11 @@ UMLAttributeList UMLClassifier::getAttributeListStatic(Visibility::Enum scope) c
  * @param n   The name of the operation to find.
  * @return    The list of objects found; will be empty if none found.
  */
-UMLOperationList UMLClassifier::findOperations(const QString &n)
+UMLOperationList UMLClassifier::findOperations(const QString &n) const
 {
     const bool caseSensitive = UMLApp::app()->activeLanguageIsCaseSensitive();
     UMLOperationList list;
-    foreach (UMLObject*  obj, m_List) {
-        uIgnoreZeroPointer(obj);
+    foreach (UMLObject*  obj, subordinates()) {
         if (obj->baseType() != UMLObject::ot_Operation)
             continue;
         UMLOperation *op = obj->asUMLOperation();
@@ -549,7 +542,8 @@ UMLOperationList UMLClassifier::findOperations(const QString &n)
  * @param considerAncestors   flag whether the ancestors should be considered during search
  * @return                    the found child object or NULL
  */
-UMLObject* UMLClassifier::findChildObjectById(Uml::ID::Type id, bool considerAncestors /* =false */)
+UMLObject* UMLClassifier::findChildObjectById(Uml::ID::Type id,
+                                              bool considerAncestors /* =false */) const
 {
     UMLObject *o = UMLCanvasObject::findChildObjectById(id);
     if (o) {
@@ -573,7 +567,7 @@ UMLObject* UMLClassifier::findChildObjectById(Uml::ID::Type id, bool considerAnc
  * @param type   The ClassifierType to seek.
  * @return       List of UMLClassifiers that inherit from us.
  */
-UMLClassifierList UMLClassifier::findSubClassConcepts (ClassifierType type)
+UMLClassifierList UMLClassifier::findSubClassConcepts (ClassifierType type) const
 {
     UMLClassifierList list = getSubClasses();
     UMLAssociationList rlist = getRealizations();
@@ -612,7 +606,7 @@ UMLClassifierList UMLClassifier::findSubClassConcepts (ClassifierType type)
  * @param type   The ClassifierType to seek.
  * @return       List of UMLClassifiers we inherit from.
  */
-UMLClassifierList UMLClassifier::findSuperClassConcepts (ClassifierType type)
+UMLClassifierList UMLClassifier::findSuperClassConcepts (ClassifierType type) const
 {
     UMLClassifierList list = getSuperClasses();
     UMLAssociationList rlist = getRealizations();
@@ -641,14 +635,6 @@ UMLClassifierList UMLClassifier::findSuperClassConcepts (ClassifierType type)
     return parentConcepts;
 }
 
-/**
- * Overloaded '==' operator.
- */
-bool UMLClassifier::operator==(const UMLClassifier & rhs) const
-{
-    return UMLCanvasObject::operator==(rhs);
-}
-
 /**
  * Copy the internal presentation of this object into the new
  * object.
@@ -659,7 +645,10 @@ void UMLClassifier::copyInto(UMLObject *lhs) const
     UMLCanvasObject::copyInto(target);
     target->setBaseType(m_BaseType);
     // CHECK: association property m_pClassAssoc is not copied
-    m_List.copyInto(&(target->m_List));
+    subordinates().copyInto(&(target->subordinates()));
+    foreach(UMLObject *o, target->subordinates()) {
+        o->setUMLParent(target);
+    }
 }
 
 /**
@@ -672,6 +661,25 @@ UMLObject* UMLClassifier::clone() const
     return clone;
 }
 
+/**
+ * override setting name of classifier
+ * @param strName name to set
+ */
+void UMLClassifier::setNameCmd(const QString &strName)
+{
+    if (UMLApp::app()->activeLanguage() == Uml::ProgrammingLanguage::Cpp ||
+            UMLApp::app()->activeLanguage() == Uml::ProgrammingLanguage::CSharp ||
+            UMLApp::app()->activeLanguage() == Uml::ProgrammingLanguage::Java) {
+        foreach(UMLOperation *op, getOpList()) {
+            if (op->isConstructorOperation())
+                op->setNameCmd(strName);
+            if (op->isDestructorOperation())
+                op->setNameCmd(QLatin1String("~") + strName);
+        }
+    }
+    UMLObject::setNameCmd(strName);
+}
+
 /**
  * Needs to be called after all UML objects are loaded from file.
  * Calls the parent resolveRef(), and calls resolveRef() on all
@@ -684,11 +692,10 @@ bool UMLClassifier::resolveRef()
 {
     bool success = UMLPackage::resolveRef();
     // Using reentrant iteration is a bare necessity here:
-    foreach (UMLObject* obj, m_List) {
-        uIgnoreZeroPointer(obj);
+    foreach (UMLObject* obj, subordinates()) {
         /**** For reference, here is the non-reentrant iteration scheme -
               DO NOT USE THIS !
-        for (UMLObject *obj = m_List.first(); obj; obj = m_List.next())
+        for (UMLObject *obj = subordinates().first(); obj; obj = subordinates().next())
          {  ....  }
          ****/
         if (obj->resolveRef()) {
@@ -716,7 +723,7 @@ bool UMLClassifier::resolveRef()
 /**
  * Reimplemented from UMLObject.
  */
-bool UMLClassifier::acceptAssociationType(AssociationType::Enum type)
+bool UMLClassifier::acceptAssociationType(AssociationType::Enum type) const
 {
     switch(type)
     {
@@ -802,14 +809,14 @@ UMLAttribute* UMLClassifier::createAttribute(const QString &name,
 
 UMLAttribute* UMLClassifier::addAttribute(const QString &name, Uml::ID::Type id /* = Uml::id_None */)
 {
-    foreach (UMLObject* obj, m_List) {
+    foreach (UMLObject* obj, subordinates()) {
         uIgnoreZeroPointer(obj);
         if (obj->baseType() == UMLObject::ot_Attribute && obj->name() == name)
             return obj->asUMLAttribute();
     }
     Uml::Visibility::Enum scope = Settings::optionState().classState.defaultAttributeScope;
     UMLAttribute *a = new UMLAttribute(this, name, id, scope);
-    m_List.append(a);
+    subordinates().append(a);
     emit attributeAdded(a);
     UMLObject::emitModified();
     connect(a, SIGNAL(modified()), this, SIGNAL(modified()));
@@ -834,7 +841,7 @@ UMLAttribute* UMLClassifier::addAttribute(const QString &name, UMLObject *type,
     if (type) {
         a->setType(type);
     }
-    m_List.append(a);
+    subordinates().append(a);
     emit attributeAdded(a);
     UMLObject::emitModified();
     connect(a, SIGNAL(modified()), this, SIGNAL(modified()));
@@ -859,11 +866,11 @@ bool UMLClassifier::addAttribute(UMLAttribute* att, IDChangeLog* log /* = 0 */,
     Q_ASSERT(att);
     if (findChildObject(att->name()) == 0) {
         att->setParent(this);
-        if (position >= 0 && position < (int)m_List.count()) {
-            m_List.insert(position, att);
+        if (position >= 0 && position < (int)subordinates().count()) {
+            subordinates().insert(position, att);
         }
         else {
-            m_List.append(att);
+            subordinates().append(att);
         }
         emit attributeAdded(att);
         UMLObject::emitModified();
@@ -885,40 +892,21 @@ bool UMLClassifier::addAttribute(UMLAttribute* att, IDChangeLog* log /* = 0 */,
  */
 int UMLClassifier::removeAttribute(UMLAttribute* att)
 {
-    if (!m_List.removeAll(att)) {
-        uDebug() << "cannot find att given in list";
+    if (!subordinates().removeAll(att)) {
+        logDebug0("UMLClassifier::removeAttribute cannot find att given in list");
         return -1;
     }
     // note that we don't delete the attribute, just remove it from the Classifier
     disconnect(att, SIGNAL(modified()), this, SIGNAL(modified()));
     emit attributeRemoved(att);
     UMLObject::emitModified();
-    return m_List.count();
-}
-
-/**
- * Sets the UMLAssociation for which this class shall act as an
- * association class.
- */
-void UMLClassifier::setClassAssoc(UMLAssociation *assoc)
-{
-    m_pClassAssoc = assoc;
-}
-
-/**
- * Returns the UMLAssociation for which this class acts as an
- * association class. Returns NULL if this class does not act
- * as an association class.
- */
-UMLAssociation *UMLClassifier::getClassAssoc() const
-{
-    return m_pClassAssoc;
+    return subordinates().count();
 }
 
 /**
  * Return true if this classifier has abstract operations.
  */
-bool UMLClassifier::hasAbstractOps ()
+bool UMLClassifier::hasAbstractOps () const
 {
     UMLOperationList opl(getOpList());
     foreach(UMLOperation *op, opl) {
@@ -935,7 +923,7 @@ bool UMLClassifier::hasAbstractOps ()
  *
  * @return   The number of operations for the Classifier.
  */
-int UMLClassifier::operations()
+int UMLClassifier::operations() const
 {
     return getOpList().count();
 }
@@ -948,10 +936,10 @@ int UMLClassifier::operations()
  * @param alreadyTraversed   internal used object to avoid recursive loops
  * @return   The list of operations for the Classifier.
  */
-UMLOperationList UMLClassifier::getOpList(bool includeInherited, UMLClassifierSet *alreadyTraversed)
+UMLOperationList UMLClassifier::getOpList(bool includeInherited, UMLClassifierSet *alreadyTraversed) const
 {
     UMLOperationList ops;
-    foreach (UMLObject* li, m_List) {
+    foreach (UMLObject* li, subordinates()) {
         uIgnoreZeroPointer(li);
         if (li->baseType() == ot_Operation) {
             ops.append(li->asUMLOperation());
@@ -971,7 +959,8 @@ UMLOperationList UMLClassifier::getOpList(bool includeInherited, UMLClassifierSe
         UMLClassifierList parents = findSuperClassConcepts();
         foreach(UMLClassifier *c, parents) {
             if (alreadyTraversed->contains(c)) {
-                uError() << "class " << c->name() << " is starting a dependency loop!";
+                logError2("UMLClassifier::getOpList(%1) : class %2 is starting a dependency loop!",
+                          name(), c->name());
                 continue;
             }
             // get operations for each parent by recursive call
@@ -1001,7 +990,7 @@ UMLOperationList UMLClassifier::getOpList(bool includeInherited, UMLClassifierSe
 }
 
 /**
- * Returns the entries in m_List that are of the requested type.
+ * Returns the entries in subordinates that are of the requested type.
  * If the requested type is UMLObject::ot_UMLObject then all entries
  * are returned.
  * @param ot   the requested object type
@@ -1010,12 +999,14 @@ UMLOperationList UMLClassifier::getOpList(bool includeInherited, UMLClassifierSe
 UMLClassifierListItemList UMLClassifier::getFilteredList(UMLObject::ObjectType ot) const
 {
     UMLClassifierListItemList resultList;
-    foreach (UMLObject* o, m_List) {
+    foreach (UMLObject* o, subordinates()) {
         uIgnoreZeroPointer(o);
         if (!o || o->baseType() == UMLObject::ot_Association) {
             continue;
         }
         UMLClassifierListItem *listItem = o->asUMLClassifierListItem();
+        if (!listItem)
+            continue;
         if (ot == UMLObject::ot_UMLObject || listItem->baseType() == ot) {
             resultList.append(listItem);
         }
@@ -1038,7 +1029,7 @@ UMLTemplate* UMLClassifier::addTemplate(const QString &name, Uml::ID::Type id)
         return templt;
     }
     templt = new UMLTemplate(this, name, id);
-    m_List.append(templt);
+    subordinates().append(templt);
     emit templateAdded(templt);
     UMLObject::emitModified();
     connect(templt, SIGNAL(modified()), this, SIGNAL(modified()));
@@ -1058,7 +1049,7 @@ bool UMLClassifier::addTemplate(UMLTemplate* newTemplate, IDChangeLog* log /* =
     QString name = newTemplate->name();
     if (findChildObject(name) == 0) {
         newTemplate->setParent(this);
-        m_List.append(newTemplate);
+        subordinates().append(newTemplate);
         emit templateAdded(newTemplate);
         UMLObject::emitModified();
         connect(newTemplate, SIGNAL(modified()), this, SIGNAL(modified()));
@@ -1087,11 +1078,11 @@ bool UMLClassifier::addTemplate(UMLTemplate* templt, int position)
     QString name = templt->name();
     if (findChildObject(name) == 0) {
         templt->setParent(this);
-        if (position >= 0 && position <= (int)m_List.count()) {
-            m_List.insert(position, templt);
+        if (position >= 0 && position <= (int)subordinates().count()) {
+            subordinates().insert(position, templt);
         }
         else {
-            m_List.append(templt);
+            subordinates().append(templt);
         }
         emit templateAdded(templt);
         UMLObject::emitModified();
@@ -1111,14 +1102,14 @@ bool UMLClassifier::addTemplate(UMLTemplate* templt, int position)
  */
 int UMLClassifier::removeTemplate(UMLTemplate* umltemplate)
 {
-    if (!m_List.removeAll(umltemplate)) {
-        uWarning() << "cannot find att given in list";
+    if (!subordinates().removeAll(umltemplate)) {
+        logWarn1("UMLClassifier::removeTemplate(%1) : cannot find att given in list", name());
         return -1;
     }
     emit templateRemoved(umltemplate);
     UMLObject::emitModified();
     disconnect(umltemplate, SIGNAL(modified()), this, SIGNAL(modified()));
-    return m_List.count();
+    return subordinates().count();
 }
 
 /**
@@ -1126,7 +1117,7 @@ int UMLClassifier::removeTemplate(UMLTemplate* umltemplate)
  * @param name   the template name
  * @return       the found template or 0
  */
-UMLTemplate *UMLClassifier::findTemplate(const QString& name)
+UMLTemplate *UMLClassifier::findTemplate(const QString& name) const
 {
     UMLTemplateList templParams = getTemplateList();
     foreach (UMLTemplate *templt, templParams) {
@@ -1142,7 +1133,7 @@ UMLTemplate *UMLClassifier::findTemplate(const QString& name)
  *
  * @return  The number of templates for the class.
  */
-int UMLClassifier::templates()
+int UMLClassifier::templates() const
 {
     UMLClassifierListItemList tempList = getFilteredList(UMLObject::ot_Template);
     return tempList.count();
@@ -1158,7 +1149,7 @@ int UMLClassifier::templates()
 UMLTemplateList UMLClassifier::getTemplateList() const
 {
     UMLTemplateList templateList;
-    foreach (UMLObject* listItem, m_List) {
+    foreach (UMLObject* listItem, subordinates()) {
         uIgnoreZeroPointer(listItem);
         if (listItem->baseType() == UMLObject::ot_Template) {
             templateList.append(listItem->asUMLTemplate());
@@ -1172,23 +1163,25 @@ UMLTemplateList UMLClassifier::getTemplateList() const
  * Ownership of the item is passed to the caller.
  *
  * @param item   Subordinate item to take.
- * @return       Index in m_List of the item taken.
- *               Return -1 if the item is not found in m_List.
+ * @return       Index in subordinates of the item taken.
+ *               Return -1 if the item is not found in subordinates.
  */
 int UMLClassifier::takeItem(UMLClassifierListItem *item)
 {
     QString buf;
-    foreach (UMLObject* currentAtt, m_List) {
+    foreach (UMLObject* currentAtt, subordinates()) {
         uIgnoreZeroPointer(currentAtt);
+#if 0
         QString txt = currentAtt->name();
         if (txt.isEmpty()) {
            txt = QLatin1String("Type-") + QString::number((int) currentAtt->baseType());
         }
+#endif
         buf.append(QLatin1Char(' ') + currentAtt->name());
     }
-    uDebug() << "  UMLClassifier::takeItem (before): m_List is " << buf;
+    logDebug1("UMLClassifier::takeItem (before): subordinates() is %1", buf);
 
-    int index = m_List.indexOf(item);
+    int index = subordinates().indexOf(item);
     if (index == -1) {
         return -1;
     }
@@ -1200,7 +1193,7 @@ int UMLClassifier::takeItem(UMLClassifierListItem *item)
             break;
         }
         case UMLObject::ot_Attribute: {
-            UMLAttribute *retval = m_List.takeAt(index)->asUMLAttribute();
+            UMLAttribute *retval = subordinates().takeAt(index)->asUMLAttribute();
             if (retval) {
                 emit attributeRemoved(retval);
                 UMLObject::emitModified();
@@ -1210,7 +1203,7 @@ int UMLClassifier::takeItem(UMLClassifierListItem *item)
             break;
         }
         case UMLObject::ot_Template: {
-            UMLTemplate *templt = m_List.takeAt(index)->asUMLTemplate();
+            UMLTemplate *templt = subordinates().takeAt(index)->asUMLTemplate();
             if (templt) {
                 emit templateRemoved(templt);
                 UMLObject::emitModified();
@@ -1220,7 +1213,7 @@ int UMLClassifier::takeItem(UMLClassifierListItem *item)
             break;
         }
         case UMLObject::ot_EnumLiteral: {
-            UMLEnumLiteral *el = m_List.takeAt(index)->asUMLEnumLiteral();
+            UMLEnumLiteral *el = subordinates().takeAt(index)->asUMLEnumLiteral();
             if (el) {
                 UMLEnum *e = this->asUMLEnum();
                 e->signalEnumLiteralRemoved(el);
@@ -1231,7 +1224,7 @@ int UMLClassifier::takeItem(UMLClassifierListItem *item)
             break;
         }
         case UMLObject::ot_EntityAttribute: {
-            UMLEntityAttribute* el = m_List.takeAt(index)->asUMLEntityAttribute();
+            UMLEntityAttribute* el = subordinates().takeAt(index)->asUMLEntityAttribute();
             if (el) {
                 UMLEntity *e = this->asUMLEntity();
                 e->signalEntityAttributeRemoved(el);
@@ -1252,7 +1245,7 @@ int UMLClassifier::takeItem(UMLClassifierListItem *item)
  * Return true if this classifier has associations.
  * @return   true if classifier has associations
  */
-bool UMLClassifier::hasAssociations()
+bool UMLClassifier::hasAssociations() const
 {
     return getSpecificAssocs(AssociationType::Association).count() > 0
             || getAggregations().count() > 0
@@ -1263,7 +1256,7 @@ bool UMLClassifier::hasAssociations()
 /**
  * Return true if this classifier has attributes.
  */
-bool UMLClassifier::hasAttributes()
+bool UMLClassifier::hasAttributes() const
 {
     return getAttributeList(Uml::Visibility::Public).count() > 0
             || getAttributeList(Uml::Visibility::Protected).count() > 0
@@ -1276,7 +1269,7 @@ bool UMLClassifier::hasAttributes()
 /**
  * Return true if this classifier has static attributes.
  */
-bool UMLClassifier::hasStaticAttributes()
+bool UMLClassifier::hasStaticAttributes() const
 {
     return getAttributeListStatic(Uml::Visibility::Public).count() > 0
             || getAttributeListStatic(Uml::Visibility::Protected).count() > 0
@@ -1286,7 +1279,7 @@ bool UMLClassifier::hasStaticAttributes()
 /**
  * Return true if this classifier has accessor methods.
  */
-bool UMLClassifier::hasAccessorMethods()
+bool UMLClassifier::hasAccessorMethods() const
 {
     return hasAttributes() || hasAssociations();
 }
@@ -1294,15 +1287,15 @@ bool UMLClassifier::hasAccessorMethods()
 /**
  * Return true if this classifier has operation methods.
  */
-bool UMLClassifier::hasOperationMethods()
+bool UMLClassifier::hasOperationMethods() const
 {
-    return getOpList().last() ? true : false;
+    return getOpList().count() > 0 ? true : false;
 }
 
 /**
  * Return true if this classifier has methods.
  */
-bool UMLClassifier::hasMethods()
+bool UMLClassifier::hasMethods() const
 {
     return hasOperationMethods() || hasAccessorMethods();
 }
@@ -1314,7 +1307,7 @@ bool UMLClassifier::hasMethods()
 /**
  * Return true if this classifier has vector fields.
  */
-bool UMLClassifier::hasVectorFields()
+bool UMLClassifier::hasVectorFields() const
 {
     return hasAssociations();
 }
@@ -1322,7 +1315,7 @@ bool UMLClassifier::hasVectorFields()
 /**
  * Return the list of unidirectional association that should show up in the code
  */
-UMLAssociationList  UMLClassifier::getUniAssociationToBeImplemented()
+UMLAssociationList  UMLClassifier::getUniAssociationToBeImplemented() const
 {
     UMLAssociationList associations = getSpecificAssocs(AssociationType::UniAssociation);
     UMLAssociationList uniAssocListToBeImplemented;
@@ -1338,7 +1331,7 @@ UMLAssociationList  UMLClassifier::getUniAssociationToBeImplemented()
             bool found = false;
             //make sure that an attribute with the same name doesn't already exist
             foreach (UMLAttribute *at, atl) {
-                uIgnoreZeroPointer(a);
+                uIgnoreZeroPointer(at);
                 if (at->name() == roleNameB) {
                     found = true;
                     break;
@@ -1358,80 +1351,104 @@ UMLAssociationList  UMLClassifier::getUniAssociationToBeImplemented()
  * Saves possible template parameters, generalizations, attributes,
  * operations, and contained objects to the given QDomElement.
  */
-void UMLClassifier::saveToXMI(QDomDocument & qDoc, QDomElement & qElement)
+void UMLClassifier::saveToXMI(QXmlStreamWriter& writer)
 {
     QString tag;
     switch (m_BaseType) {
         case UMLObject::ot_Class:
-            tag = QLatin1String("UML:Class");
+            tag = QLatin1String("Class");
             break;
         case UMLObject::ot_Interface:
-            tag = QLatin1String("UML:Interface");
+            tag = QLatin1String("Interface");
             break;
         case UMLObject::ot_Package:
-            UMLPackage::saveToXMI(qDoc, qElement);
+            UMLPackage::saveToXMI(writer);
             return;
             break;
         default:
-            uError() << "internal error: basetype is " << m_BaseType;
+            logError2("UMLClassifier::saveToXMI(%1) internal error: basetype is %2",
+                      name(), m_BaseType);
             return;
     }
-    QDomElement classifierElement = UMLObject::save(tag, qDoc);
+    UMLObject::save1(writer, tag);
 
     //save templates
     UMLClassifierListItemList list = getFilteredList(UMLObject::ot_Template);
     if (list.count()) {
-        QDomElement tmplElement = qDoc.createElement(QLatin1String("UML:ModelElement.templateParameter"));
+        if (! Settings::optionState().generalState.uml2) {
+            writer.writeStartElement(QLatin1String("UML:ModelElement.templateParameter"));
+        }
         foreach (UMLClassifierListItem *tmpl, list) {
-            tmpl->saveToXMI(qDoc, tmplElement);
+            tmpl->saveToXMI(writer);
+        }
+        if (! Settings::optionState().generalState.uml2) {
+            writer.writeEndElement();
         }
-        classifierElement.appendChild(tmplElement);
     }
 
     //save generalizations (we are the subclass, the other end is the superclass)
     UMLAssociationList generalizations = getSpecificAssocs(AssociationType::Generalization);
     if (generalizations.count()) {
-        QDomElement genElement = qDoc.createElement(QLatin1String("UML:GeneralizableElement.generalization"));
-        foreach (UMLAssociation *a, generalizations) {
+        if (! Settings::optionState().generalState.uml2) {
+            writer.writeStartElement(QLatin1String("UML:GeneralizableElement.generalization"));
+        }
+        foreach (UMLAssociation *asso, generalizations) {
             // We are the subclass if we are at the role A end.
-            if (m_nId != a->getObjectId(RoleType::A)) {
+            if (m_nId != asso->getObjectId(RoleType::A)) {
                 continue;
             }
-            QDomElement gElem = qDoc.createElement(QLatin1String("UML:Generalization"));
-            gElem.setAttribute(QLatin1String("xmi.idref"), Uml::ID::toString(a->id()));
-            genElement.appendChild(gElem);
+            if (Settings::optionState().generalState.uml2) {
+                writer.writeStartElement(QLatin1String("generalization"));
+                writer.writeAttribute(QLatin1String("xmi:type"), QLatin1String("uml:Generalization"));
+                writer.writeAttribute(QLatin1String("xmi:id"), Uml::ID::toString(asso->id()));
+                Uml::ID::Type superId = asso->getObjectId(RoleType::B);
+                writer.writeAttribute(QLatin1String("general"), Uml::ID::toString(superId));
+            } else {
+                writer.writeStartElement(QLatin1String("UML:Generalization"));
+                // CHECK - This looks wrong, we should be saving the role B ID (superclass)
+                writer.writeAttribute(QLatin1String("xmi.idref"), Uml::ID::toString(asso->id()));
+            }
+            writer.writeEndElement();
         }
-        if (genElement.hasChildNodes()) {
-            classifierElement.appendChild(genElement);
+        if (! Settings::optionState().generalState.uml2) {
+            writer.writeEndElement();
         }
     }
 
-    // save attributes
-    QDomElement featureElement = qDoc.createElement(QLatin1String("UML:Classifier.feature"));
     UMLClassifierListItemList attList = getFilteredList(UMLObject::ot_Attribute);
-    foreach (UMLClassifierListItem *pAtt, attList) {
-        pAtt->saveToXMI(qDoc, featureElement);
-    }
+    UMLOperationList          opList  = getOpList();
 
-    // save operations
-    UMLOperationList opList = getOpList();
-    foreach (UMLOperation *pOp, opList) {
-        pOp->saveToXMI(qDoc, featureElement);
-    }
-    if (featureElement.hasChildNodes()) {
-        classifierElement.appendChild(featureElement);
+    if (attList.count() || opList.count()) {
+        if (! Settings::optionState().generalState.uml2) {
+            writer.writeStartElement(QLatin1String("UML:Classifier.feature"));
+        }
+        // save attributes
+        foreach (UMLClassifierListItem *pAtt, attList) {
+            pAtt->saveToXMI(writer);
+        }
+        // save operations
+        foreach (UMLOperation *pOp, opList) {
+            pOp->saveToXMI(writer);
+        }
+        if (! Settings::optionState().generalState.uml2) {
+            writer.writeEndElement();
+        }
     }
 
     // save contained objects
     if (m_objects.count()) {
-        QDomElement ownedElement = qDoc.createElement(QLatin1String("UML:Namespace.ownedElement"));
+        if (! Settings::optionState().generalState.uml2) {
+            writer.writeStartElement(QLatin1String("UML:Namespace.ownedElement"));
+        }
         foreach (UMLObject* obj, m_objects) {
             uIgnoreZeroPointer(obj);
-            obj->saveToXMI (qDoc, ownedElement);
+            obj->saveToXMI (writer);
+        }
+        if (! Settings::optionState().generalState.uml2) {
+            writer.writeEndElement();
         }
-        classifierElement.appendChild(ownedElement);
     }
-    qElement.appendChild(classifierElement);
+    UMLObject::save1end(writer);   // from UMLObject::save1(tag)
 }
 
 /**
@@ -1452,7 +1469,8 @@ UMLClassifierListItem* UMLClassifier::makeChildObject(const QString& xmiTag)
         if (baseType() != UMLObject::ot_Class)
             return 0;
         pObject = new UMLAttribute(this);
-    } else if (UMLDoc::tagEq(xmiTag, QLatin1String("TemplateParameter"))) {
+    } else if (UMLDoc::tagEq(xmiTag, QLatin1String("TemplateParameter")) ||
+               UMLDoc::tagEq(xmiTag, QLatin1String("ClassifierTemplateParameter"))) {
         pObject = new UMLTemplate(this);
     }
     return pObject;
@@ -1465,7 +1483,7 @@ UMLClassifierListItem* UMLClassifier::makeChildObject(const QString& xmiTag)
  * Child classes can override the loadFromXMI() method
  * to load its additional tags.
  */
-bool UMLClassifier::load(QDomElement& element)
+bool UMLClassifier::load1(QDomElement& element)
 {
     UMLClassifierListItem *child = 0;
     bool totalSuccess = true;
@@ -1481,7 +1499,7 @@ bool UMLClassifier::load(QDomElement& element)
                 UMLDoc::tagEq(tag, QLatin1String("Namespace.ownedElement")) ||
                 UMLDoc::tagEq(tag, QLatin1String("Element.ownedElement")) ||  // Embarcadero's Describe
                 UMLDoc::tagEq(tag, QLatin1String("Namespace.contents"))) {
-            load(element);
+            load1(element);
             // Not evaluating the return value from load()
             // because we want a best effort.
 
@@ -1493,23 +1511,24 @@ bool UMLClassifier::load(QDomElement& element)
                         break;
                     case UMLObject::ot_Operation:
                         if (! addOperation(child->asUMLOperation())) {
-                            uError() << "error from addOperation(op)";
+                            logError1("UMLClassifier::load1(%1) : error from addOperation(op)", name());
                             delete child;
                             totalSuccess = false;
                         }
                         break;
                     case UMLObject::ot_Attribute:
+                    case UMLObject::ot_InstanceAttribute:
                         addAttribute(child->asUMLAttribute());
                         break;
                     default:
                         break;
                 }
             } else {
-                uWarning() << "failed to load " << tag;
+                logWarn2("UMLClassifier::load1(%1): failed to load %2", name(), tag);
                 delete child;
                 totalSuccess = false;
             }
-        } else if (!Model_Utils::isCommonXMIAttribute(tag)) {
+        } else if (!Model_Utils::isCommonXMI1Attribute(tag)) {
             UMLObject *pObject = Object_Factory::makeObjectFromXMI(tag, stereotype);
             if (pObject == 0) {
                 // Not setting totalSuccess to false
