@@ -1,6 +1,6 @@
 /*
     SPDX-License-Identifier: GPL-2.0-or-later
-    SPDX-FileCopyrightText: 2004-2021 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
+    SPDX-FileCopyrightText: 2004-2022 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
 */
 
 #include "widgetbase.h"
@@ -31,6 +31,8 @@
 #include <QPointer>
 #include <QXmlStreamWriter>
 
+DEBUG_REGISTER(WidgetBase)
+
 static unsigned eventCnt = 0;
 
 void QGraphicsObjectWrapper::setSelected(bool state)
@@ -42,9 +44,11 @@ void QGraphicsObjectWrapper::setSelected(bool state)
     if (wb)
         info = wb->name();
     if (info.isEmpty()) {
-        DEBUG(DBG_SRC) << ++eventCnt << " new state=" << state << ", fromItemChange=" << m_calledFromItemChange << " " << this;
-    } else {
-        DEBUG(DBG_SRC) << ++eventCnt << " new state=" << state << ", fromItemChange=" << m_calledFromItemChange << " " << info;
+        DEBUG()
+            << ++eventCnt << " new state=" << state << ", fromItemChange=" << m_calledFromItemChange << " " << this;
+    } else {   // @todo convert DEBUG() stream to logDebug3 macro (uml.h)
+        DEBUG()
+            << ++eventCnt << " new state=" << state << ", fromItemChange=" << m_calledFromItemChange << " " << info;
     }
     m_calledFromItemChange = false;
 }
@@ -65,6 +69,7 @@ QVariant QGraphicsObjectWrapper::itemChange(GraphicsItemChange change, const QVa
  * @param scene   The view to be displayed on.
  * @param type    The WidgetType to construct.  This must be set to the appropriate
  *                value by the constructors of inheriting classes.
+ * @param id      The XMI ID to use. The value Uml::ID::None will trigger generation of a new ID.
  */
 WidgetBase::WidgetBase(UMLScene *scene, WidgetType type, Uml::ID::Type id)
   : QGraphicsObjectWrapper(),
@@ -105,7 +110,7 @@ WidgetBase::WidgetBase(UMLScene *scene, WidgetType type, Uml::ID::Type id)
         setLineWidth(optionState.uiState.lineWidth);
         m_font = optionState.uiState.font;
     } else {
-        uError() << "WidgetBase constructor: SERIOUS PROBLEM - m_scene is NULL";
+        logError0("WidgetBase constructor: SERIOUS PROBLEM - m_scene is NULL");
     }
 }
 
@@ -208,8 +213,8 @@ void WidgetBase::setID(Uml::ID::Type id)
 {
     if (m_umlObject) {
         if (m_umlObject->id() != Uml::ID::None)
-            uWarning() << "changing old UMLObject " << Uml::ID::toString(m_umlObject->id())
-                << " to " << Uml::ID::toString(id);
+            logWarn2("WidgetBase::setID changing old UMLObject %1 to %2",
+                     Uml::ID::toString(m_umlObject->id()), Uml::ID::toString(id));
         m_umlObject->setID(id);
     }
     m_nId = id;
@@ -285,7 +290,7 @@ QString WidgetBase::documentation() const
  *
  * @return false if documentation is empty
  */
-bool WidgetBase::hasDocumentation()
+bool WidgetBase::hasDocumentation() const
 {
     if (m_umlObject)
         return m_umlObject->hasDoc();
@@ -308,7 +313,7 @@ void WidgetBase::setDocumentation(const QString& doc)
 
 /**
  * Gets the name from the corresponding UMLObject if this widget has an
- * underlying UMLObject; if it does not, then it returns the local
+ * underlying UMLObject; if it does not then it returns the local
  * m_Text (notably the case for FloatingTextWidget.)
  *
  * @return the currently set name
@@ -423,7 +428,7 @@ void WidgetBase::setLineWidth(uint width)
  *
  * @return True if fill color is used
  */
-bool WidgetBase::useFillColor()
+bool WidgetBase::useFillColor() const
 {
     return m_useFillColor;
 }
@@ -564,7 +569,7 @@ void WidgetBase::setFont(const QFont& font)
  * Return state of auto resize property
  * @return the auto resize state
  */
-bool WidgetBase::autoResize()
+bool WidgetBase::autoResize() const
 {
     return m_autoResize;
 }
@@ -582,7 +587,7 @@ void WidgetBase::setAutoResize(bool state)
  * Return changes state property
  * @return the changes shape state
  */
-bool WidgetBase::changesShape()
+bool WidgetBase::changesShape() const
 {
     return m_changesShape;
 }
@@ -621,7 +626,7 @@ bool WidgetBase::showPropertiesDialog()
  *
  * @param writer The QXmlStreamWriter to write to.
  */
-void WidgetBase::saveToXMI1(QXmlStreamWriter& writer)
+void WidgetBase::saveToXMI(QXmlStreamWriter& writer)
  {
     writer.writeAttribute(QLatin1String("xmi.id"), Uml::ID::toString(id()));
     // Unique identifier for widget (todo: id() should be unique, new attribute
@@ -690,7 +695,8 @@ bool WidgetBase::activate(IDChangeLog* changeLog)
     if (widgetHasUMLObject(baseType()) && m_umlObject == nullptr) {
         m_umlObject =  UMLApp::app()->document()->findObjectById(m_nId);
         if (m_umlObject == nullptr) {
-            uError() << "cannot find UMLObject with id=" << Uml::ID::toString(m_nId);
+            logError1("WidgetBase::activate: cannot find UMLObject with id=%1",
+                      Uml::ID::toString(m_nId));
             return false;
         }
     }
@@ -727,7 +733,7 @@ void WidgetBase::removeAssoc(AssociationWidget *pAssoc)
  *
  * @todo Add support to load older version.
  */
-bool WidgetBase::loadFromXMI1(QDomElement& qElement)
+bool WidgetBase::loadFromXMI(QDomElement& qElement)
 {
     QString id = qElement.attribute(QLatin1String("xmi.id"), QLatin1String("-1"));
     m_nId = Uml::ID::fromString(id);
@@ -792,8 +798,8 @@ bool WidgetBase::loadFromXMI1(QDomElement& qElement)
         newFont.fromString(font);
         m_font = newFont;
     } else {
-        uWarning() << "Using default font " << m_font.toString()
-                   << " for widget with xmi.id " << Uml::ID::toString(m_nId);
+        logWarn2("WidgetBase::loadFromXMI: Using default font %1 for widget with xmi.id %2", 
+                 m_font.toString(), Uml::ID::toString(m_nId));
     }
     QString autoResize = qElement.attribute(QLatin1String("autoresize"), QLatin1String("1"));
     m_autoResize = (bool)autoResize.toInt();
@@ -806,6 +812,9 @@ bool WidgetBase::loadFromXMI1(QDomElement& qElement)
  */
 WidgetBase& WidgetBase::operator=(const WidgetBase& other)
 {
+    if (&other == this)
+        return *this;
+
     m_baseType = other.m_baseType;
     m_scene = other.m_scene;
     m_umlObject = other.m_umlObject;
@@ -844,6 +853,7 @@ void WidgetBase::setRect(const QRectF& rect)
 {
     if (m_rect == rect)
         return;
+    logDebug3("WidgetBase::setRect(%1) : setting w=%2, h=%3", name(), rect.width(), rect.height());
     prepareGeometryChange();
     m_rect = rect;
     update();
@@ -863,7 +873,7 @@ void WidgetBase::setRect(qreal x, qreal y, qreal width, qreal height)
  */
 QRectF WidgetBase::boundingRect() const
 {
-    int halfWidth = lineWidth()/2;
+    qreal halfWidth = lineWidth() / 2.0;
     return m_rect.adjusted(-halfWidth, -halfWidth, halfWidth, halfWidth);
 }
 
@@ -918,7 +928,7 @@ void WidgetBase::paint(QPainter *painter, const QStyleOptionGraphicsItem *option
 void WidgetBase::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 {
     event->accept();
-    uDebug() << "widget = " << name() << " / type = " << baseTypeStr();
+    logDebug2("WidgetBase::contextMenuEvent: widget = %1 / type = %2", name(), baseTypeStr());
 
     UMLScene *scene = umlScene();
 
@@ -993,7 +1003,7 @@ void WidgetBase::slotMenuSelection(QAction *trigger)
         } else if (wt == WidgetBase::wt_Object) {
             m_umlObject->showPropertiesDialog();
         } else {
-            uWarning() << "making properties dialog for unknown widget type";
+            logWarn1("WidgetBase::slotMenuSelection: making properties dialog for unknown widget type %1", sel);
         }
         break;
 
@@ -1210,7 +1220,8 @@ void WidgetBase::slotMenuSelection(QAction *trigger)
         umlScene()->alignHorizontalDistribute();
         break;
     default:
-        uDebug() << "MenuType " << ListPopupMenu::toString(sel) << " not implemented";
+        logDebug1("WidgetBase::slotMenuSelection: MenuType %1 not implemented",
+                  ListPopupMenu::toString(sel));
         break;
     }
 }
@@ -1328,7 +1339,7 @@ QString WidgetBase::toI18nString(WidgetType wt)
         break;
     default:
         name = QLatin1String("<unknown> &name:");
-        uWarning() << "unknown widget type";
+        logWarn1("WidgetBase::toI18nString: unknown widget type %1", wt);
         break;
     }
     return name;
@@ -1436,7 +1447,7 @@ Icon_Utils::IconType WidgetBase::toIcon(WidgetBase::WidgetType wt)
         break;
     default:
         icon = Icon_Utils::it_Home;
-        uWarning() << "unknown widget type";
+        logWarn1("WidgetBase::toIcon: unknown widget type %1", wt);
         break;
     }
     return icon;
@@ -1471,35 +1482,69 @@ Icon_Utils::IconType WidgetBase::toIcon(WidgetBase::WidgetType wt)
 #include  "statewidget.h"
 #include  "usecasewidget.h"
 
-ActivityWidget* WidgetBase::asActivityWidget() { return dynamic_cast<ActivityWidget* >(this); }
-ActorWidget* WidgetBase::asActorWidget() { return dynamic_cast<ActorWidget* >(this); }
-ArtifactWidget* WidgetBase::asArtifactWidget() { return dynamic_cast<ArtifactWidget* >(this); }
-AssociationWidget* WidgetBase::asAssociationWidget() { return dynamic_cast<AssociationWidget* >(this); }
-BoxWidget* WidgetBase::asBoxWidget() { return dynamic_cast<BoxWidget* >(this); }
-CategoryWidget* WidgetBase::asCategoryWidget() { return dynamic_cast<CategoryWidget* >(this); }
-ClassifierWidget* WidgetBase::asClassifierWidget() { return dynamic_cast<ClassifierWidget* >(this); }
+ActivityWidget*         WidgetBase::asActivityWidget()         { return dynamic_cast<ActivityWidget* >       (this); }
+ActorWidget*            WidgetBase::asActorWidget()            { return dynamic_cast<ActorWidget* >          (this); }
+ArtifactWidget*         WidgetBase::asArtifactWidget()         { return dynamic_cast<ArtifactWidget* >       (this); }
+AssociationWidget*      WidgetBase::asAssociationWidget()      { return dynamic_cast<AssociationWidget* >    (this); }
+BoxWidget*              WidgetBase::asBoxWidget()              { return dynamic_cast<BoxWidget* >            (this); }
+CategoryWidget*         WidgetBase::asCategoryWidget()         { return dynamic_cast<CategoryWidget* >       (this); }
+ClassifierWidget*       WidgetBase::asClassifierWidget()       { return dynamic_cast<ClassifierWidget* >     (this); }
 CombinedFragmentWidget* WidgetBase::asCombinedFragmentWidget() { return dynamic_cast<CombinedFragmentWidget*>(this); }
-ComponentWidget* WidgetBase::asComponentWidget() { return dynamic_cast<ComponentWidget* >(this); }
-DatatypeWidget* WidgetBase::asDatatypeWidget() { return dynamic_cast<DatatypeWidget* >(this); }
-EntityWidget* WidgetBase::asEntityWidget() { return dynamic_cast<EntityWidget* >(this); }
-EnumWidget* WidgetBase::asEnumWidget() { return dynamic_cast<EnumWidget* >(this); }
+ComponentWidget*        WidgetBase::asComponentWidget()        { return dynamic_cast<ComponentWidget* >      (this); }
+DatatypeWidget*         WidgetBase::asDatatypeWidget()         { return dynamic_cast<DatatypeWidget* >       (this); }
+EntityWidget*           WidgetBase::asEntityWidget()           { return dynamic_cast<EntityWidget* >         (this); }
+EnumWidget*             WidgetBase::asEnumWidget()             { return dynamic_cast<EnumWidget* >           (this); }
 FloatingDashLineWidget* WidgetBase::asFloatingDashLineWidget() { return dynamic_cast<FloatingDashLineWidget*>(this); }
-ForkJoinWidget* WidgetBase::asForkJoinWidget() { return dynamic_cast<ForkJoinWidget* >(this); }
-InterfaceWidget* WidgetBase::asInterfaceWidget() { return dynamic_cast<InterfaceWidget* >(this); }
-MessageWidget* WidgetBase::asMessageWidget() { return dynamic_cast<MessageWidget* >(this); }
-NodeWidget* WidgetBase::asNodeWidget() { return dynamic_cast<NodeWidget* >(this); }
-NoteWidget* WidgetBase::asNoteWidget() { return dynamic_cast<NoteWidget* >(this); }
-ObjectNodeWidget* WidgetBase::asObjectNodeWidget() { return dynamic_cast<ObjectNodeWidget* >(this); }
-ObjectWidget* WidgetBase::asObjectWidget() { return dynamic_cast<ObjectWidget* >(this); }
-PackageWidget* WidgetBase::asPackageWidget() { return dynamic_cast<PackageWidget* >(this); }
-PinWidget* WidgetBase::asPinWidget() { return dynamic_cast<PinWidget* >(this); }
-PinPortBase *WidgetBase::asPinPortBase() { return dynamic_cast<PinPortBase*>(this); }
-PortWidget* WidgetBase::asPortWidget() { return dynamic_cast<PortWidget* >(this); }
-PreconditionWidget* WidgetBase::asPreconditionWidget() { return dynamic_cast<PreconditionWidget* >(this); }
-RegionWidget* WidgetBase::asRegionWidget() { return dynamic_cast<RegionWidget* >(this); }
-SignalWidget* WidgetBase::asSignalWidget() { return dynamic_cast<SignalWidget* >(this); }
-StateWidget* WidgetBase::asStateWidget() { return dynamic_cast<StateWidget* >(this); }
-FloatingTextWidget* WidgetBase::asFloatingTextWidget() { return dynamic_cast<FloatingTextWidget* >(this); }
-//TextWidget* WidgetBase::asTextWidget() { return dynamic_cast<TextWidget* >(this); }
-UseCaseWidget* WidgetBase::asUseCaseWidget() { return dynamic_cast<UseCaseWidget* >(this); }
-UMLWidget *WidgetBase::asUMLWidget() { return dynamic_cast<UMLWidget*>(this); }
+ForkJoinWidget*         WidgetBase::asForkJoinWidget()         { return dynamic_cast<ForkJoinWidget* >       (this); }
+InterfaceWidget*        WidgetBase::asInterfaceWidget()        { return dynamic_cast<InterfaceWidget* >      (this); }
+MessageWidget*          WidgetBase::asMessageWidget()          { return dynamic_cast<MessageWidget* >        (this); }
+NodeWidget*             WidgetBase::asNodeWidget()             { return dynamic_cast<NodeWidget* >           (this); }
+NoteWidget*             WidgetBase::asNoteWidget()             { return dynamic_cast<NoteWidget* >           (this); }
+ObjectNodeWidget*       WidgetBase::asObjectNodeWidget()       { return dynamic_cast<ObjectNodeWidget* >     (this); }
+ObjectWidget*           WidgetBase::asObjectWidget()           { return dynamic_cast<ObjectWidget* >         (this); }
+PackageWidget*          WidgetBase::asPackageWidget()          { return dynamic_cast<PackageWidget* >        (this); }
+PinWidget*              WidgetBase::asPinWidget()              { return dynamic_cast<PinWidget* >            (this); }
+PinPortBase*            WidgetBase::asPinPortBase()            { return dynamic_cast<PinPortBase*>           (this); }
+PortWidget*             WidgetBase::asPortWidget()             { return dynamic_cast<PortWidget* >           (this); }
+PreconditionWidget*     WidgetBase::asPreconditionWidget()     { return dynamic_cast<PreconditionWidget* >   (this); }
+RegionWidget*           WidgetBase::asRegionWidget()           { return dynamic_cast<RegionWidget* >         (this); }
+SignalWidget*           WidgetBase::asSignalWidget()           { return dynamic_cast<SignalWidget* >         (this); }
+StateWidget*            WidgetBase::asStateWidget()            { return dynamic_cast<StateWidget* >          (this); }
+FloatingTextWidget*     WidgetBase::asFloatingTextWidget()     { return dynamic_cast<FloatingTextWidget* >   (this); }
+//TextWidget*             WidgetBase::asTextWidget()             { return dynamic_cast<TextWidget* >           (this); }
+UseCaseWidget*          WidgetBase::asUseCaseWidget()          { return dynamic_cast<UseCaseWidget* >        (this); }
+UMLWidget*              WidgetBase::asUMLWidget()              { return dynamic_cast<UMLWidget*>             (this); }
+
+const ActivityWidget*         WidgetBase::asActivityWidget()         const { return dynamic_cast<const ActivityWidget* >       (this); }
+const ActorWidget*            WidgetBase::asActorWidget()            const { return dynamic_cast<const ActorWidget* >          (this); }
+const ArtifactWidget*         WidgetBase::asArtifactWidget()         const { return dynamic_cast<const ArtifactWidget* >       (this); }
+const AssociationWidget*      WidgetBase::asAssociationWidget()      const { return dynamic_cast<const AssociationWidget* >    (this); }
+const BoxWidget*              WidgetBase::asBoxWidget()              const { return dynamic_cast<const BoxWidget* >            (this); }
+const CategoryWidget*         WidgetBase::asCategoryWidget()         const { return dynamic_cast<const CategoryWidget* >       (this); }
+const ClassifierWidget*       WidgetBase::asClassifierWidget()       const { return dynamic_cast<const ClassifierWidget* >     (this); }
+const CombinedFragmentWidget* WidgetBase::asCombinedFragmentWidget() const { return dynamic_cast<const CombinedFragmentWidget*>(this); }
+const ComponentWidget*        WidgetBase::asComponentWidget()        const { return dynamic_cast<const ComponentWidget* >      (this); }
+const DatatypeWidget*         WidgetBase::asDatatypeWidget()         const { return dynamic_cast<const DatatypeWidget* >       (this); }
+const EntityWidget*           WidgetBase::asEntityWidget()           const { return dynamic_cast<const EntityWidget* >         (this); }
+const EnumWidget*             WidgetBase::asEnumWidget()             const { return dynamic_cast<const EnumWidget* >           (this); }
+const FloatingDashLineWidget* WidgetBase::asFloatingDashLineWidget() const { return dynamic_cast<const FloatingDashLineWidget*>(this); }
+const ForkJoinWidget*         WidgetBase::asForkJoinWidget()         const { return dynamic_cast<const ForkJoinWidget* >       (this); }
+const InterfaceWidget*        WidgetBase::asInterfaceWidget()        const { return dynamic_cast<const InterfaceWidget* >      (this); }
+const MessageWidget*          WidgetBase::asMessageWidget()          const { return dynamic_cast<const MessageWidget* >        (this); }
+const NodeWidget*             WidgetBase::asNodeWidget()             const { return dynamic_cast<const NodeWidget* >           (this); }
+const NoteWidget*             WidgetBase::asNoteWidget()             const { return dynamic_cast<const NoteWidget* >           (this); }
+const ObjectNodeWidget*       WidgetBase::asObjectNodeWidget()       const { return dynamic_cast<const ObjectNodeWidget* >     (this); }
+const ObjectWidget*           WidgetBase::asObjectWidget()           const { return dynamic_cast<const ObjectWidget* >         (this); }
+const PackageWidget*          WidgetBase::asPackageWidget()          const { return dynamic_cast<const PackageWidget* >        (this); }
+const PinWidget*              WidgetBase::asPinWidget()              const { return dynamic_cast<const PinWidget* >            (this); }
+const PinPortBase*            WidgetBase::asPinPortBase()            const { return dynamic_cast<const PinPortBase*>           (this); }
+const PortWidget*             WidgetBase::asPortWidget()             const { return dynamic_cast<const PortWidget* >           (this); }
+const PreconditionWidget*     WidgetBase::asPreconditionWidget()     const { return dynamic_cast<const PreconditionWidget* >   (this); }
+const RegionWidget*           WidgetBase::asRegionWidget()           const { return dynamic_cast<const RegionWidget* >         (this); }
+const SignalWidget*           WidgetBase::asSignalWidget()           const { return dynamic_cast<const SignalWidget* >         (this); }
+const StateWidget*            WidgetBase::asStateWidget()            const { return dynamic_cast<const StateWidget* >          (this); }
+const FloatingTextWidget*     WidgetBase::asFloatingTextWidget()     const { return dynamic_cast<const FloatingTextWidget* >   (this); }
+//const TextWidget*             WidgetBase::asTextWidget()             const { return dynamic_cast<const TextWidget* >           (this); }
+const UseCaseWidget*          WidgetBase::asUseCaseWidget()          const { return dynamic_cast<const UseCaseWidget* >        (this); }
+const UMLWidget*              WidgetBase::asUMLWidget()              const { return dynamic_cast<const UMLWidget*>             (this); }
+
